/*
 * @copyright       © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/content/inc/megamenu
 *
 * @revision        $Revision: 1.11.32.1 $
 */

/*jslint plusplus: true*/
/*global define*/

define("de_epages/content/inc/megamenu", [
    "jquery",
    "ep",

    "jquery/masonry"
], function ($, ep) {
    'use strict';

    return {
        init: function (options) {
            options = $.extend({
                letterCount : 20,
                fallbackColumWidth : 200,
                minOffsetLeft: 10,
                menuDelay: 500,
                isEditorMode: false
            }, options);

            var $menu = $(options.selector),

                $main = $menu.parents('.PropertyContainer'),

                mainWidth = $main.width(),

                $activeMenu,

                $activeListEntry,

                columWidth,

                // Calculate width of menu container will be updated at hover event but is needed now for initalization
                correctContainer = function ($container) {
                    // Break if no container available
                    if (!$container || !$container.length) {
                        return;
                    }

                    // Contextual menu item for the conatiner
                    var countSecondLevel = $container.find('.ep-megamenu-second-level-element').length,

                        maxColumnCount =  Math.floor((mainWidth - (options.minOffsetLeft * 2)) / columWidth),

                        currentColumWidth = countSecondLevel < maxColumnCount ? countSecondLevel * columWidth : (maxColumnCount * columWidth),

                        // Main container properties
                        mainOuterWidth = $main.outerWidth(true),
                        mainOffsetLeft = $main.offset().left,
                        mainEndPoint = mainOuterWidth + mainOffsetLeft,

                        // Container properties
                        containerOuterWidth,
                        containerOffsetLeft,
                        containerEndPoint;

                    // Set conatiner width and rest left
                    $container.css({
                        left: '',
                        width: currentColumWidth + 'px'
                    });

                    // Read conatiner properties
                    containerOuterWidth = $container.outerWidth(true);
                    containerOffsetLeft = $container.offset().left;
                    containerEndPoint = containerOuterWidth + containerOffsetLeft;

                    // Check minimum offset
                    if (containerOffsetLeft - mainOffsetLeft < options.minOffsetLeft) {
                        $container.css({
                            left: (options.minOffsetLeft) + 'px'
                        });
                    } else if (containerEndPoint + options.minOffsetLeft > mainEndPoint) {
                        $container.css({
                            left: (containerOffsetLeft - $container.offsetParent().offset().left) - (containerEndPoint - (mainEndPoint - options.minOffsetLeft)) + 'px'
                        });
                    }
                },

                letterWidthNode,

                letterWidth,

                $items;

            if ($menu.length) {
                //create element to test current letter width
                letterWidthNode = $('<span>').appendTo('body');

                //use variety of widest letters to calculate max letter width
                letterWidthNode
                    .text('PMWXGOQ')
                    .addClass('ep-megamenu-second-level-font-check');

                letterWidth = Math.floor(letterWidthNode.width() / 7);

                letterWidthNode.remove();

                columWidth = (letterWidth ? letterWidth * options.letterCount : options.fallbackColumWidth);

                $menu.find('.ep-megamenu-second-level-element')
                    .css({
                        width: columWidth  + 'px'
                    });

                $items = $menu.children('.ep-megamenu-first-level-element');

                //menu hover
                $menu.on('mouseenter', '.ep-megamenu-first-level-element', function () {
                    var $item = $(this),
                        $container = $item.children('.ep-megamenu-container'),
                        $contentList;

                    if ($container.length) {
                        $contentList = $container.children('.ep-megamenu-second-level-group');

                        $container
                            .clearQueue()
                            .queue(function () {
                                var contentListHeight;

                                //hide last menu container
                                if ($activeListEntry) {
                                    $activeListEntry.removeClass('active-item');
                                }

                                $activeListEntry = $item;
                                $activeListEntry.addClass('active-item');

                                if ($activeMenu) {
                                    $activeMenu.clearQueue().hide();
                                }

                                $activeMenu = $container;
                                $container.show();
                                $contentList.masonry();

                                //check li height
                                $contentList
                                    .children('.ep-megamenu-second-level-element')
                                    .each(function () {
                                        var entry = $(this),
                                            entryOuterWidth = entry.outerWidth(true),
                                            entryHeight = entry.outerHeight(true),
                                            entryTopOffset = entry.position().top,
                                            visibleEntryCount = entry.find('.ep-megamenu-third-level-element:visible').length,
                                            containerHeight = $container.height(),
                                            containerMaxHeight = parseInt($container.css('max-height'), 10),
                                            i;

                                        if (entryOuterWidth !== columWidth) {
                                            entry.css({
                                                width: columWidth - (entryOuterWidth - columWidth) + 'px'
                                            });
                                        }

                                        containerHeight = containerHeight < containerMaxHeight ? containerMaxHeight : containerHeight;

                                        //hide elements until list fits into given height
                                        $contentList.masonry();

                                        for (i = 0; i < visibleEntryCount; i++) {
                                            entryTopOffset = entry.position().top;

                                            if (entryHeight + entryTopOffset > containerHeight) {
                                                entry.find('.ep-megamenu-more').show();
                                                entry.find('.ep-megamenu-third-level-element:visible').last().hide();
                                                entryHeight = entry.outerHeight(true);
                                            }
                                        }
                                    });

                                contentListHeight = $contentList.height();

                                if (contentListHeight > parseInt($container.css('max-height'), 10)) {
                                    $container.css({
                                        'max-height': contentListHeight + 'px'
                                    });
                                    $contentList.masonry();
                                }
                            });

                        //correct $container position
                        correctContainer($container);
                    }
                });

                //hover out
                $menu.on('mouseleave', '.ep-megamenu-first-level-element', function () {
                    var $item = $(this),
                        $container = $item.children('.ep-megamenu-container');

                    $container
                        .clearQueue()
                        .delay(options.menuDelay)
                        .queue(function () {
                            $container.hide();

                            if ($activeListEntry) {
                                $activeListEntry.removeClass('active-item');
                            }
                        });
                });

                $items.each(function () {
                    var $item = $(this),
                        $container = $item.children('.ep-megamenu-container'),
                        $activeMarker = $item.find('.active-marker'),
                        $itemAnchor = $item.children('a');

                    $activeMarker.css({
                        'left': (($itemAnchor.width() / 2) - 5) + 'px'
                    });

                    correctContainer($container);
                });

                //initalize masonry
                $menu.find('.ep-megamenu-second-level-group')
                    .masonry({
                        itemSelector : '.ep-megamenu-second-level-element',
                        columnWidth: columWidth
                    });
            }
            //add ActiveClass to current selected main node
            $('.Category-' + ep.config.objectId + '.NavigationElement').addClass('Active');
        }
    };
});