use Test::More tests => 29;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Iterator qw( ArrayIterator );
use DE_EPAGES::Core::API::PerlTools qw( IsA );

## no critic qw( ProhibitMagicNumbers )
RunScript( Debug => 1, Sub => sub {

    my @Array = ( 1..9 );
    my $ArrayIterator = ArrayIterator( \@Array );

    is( $ArrayIterator->count, 9, 'ArrayIterator.count' );
    my $Sum = 0;
    $Sum += $_ while <$ArrayIterator>;
    is( $Sum, 45, 'ArrayIterator' );
    is( scalar @Array, 9, 'original array is not modified' );

    is( "$ArrayIterator", 'DE_EPAGES::Core::API::Iterator(count=9)', 'Overload ""' );

    my $i = 1;
    my $TestIterator = DE_EPAGES::Core::API::Iterator->new(
        Pull => sub {
            ok( IsA( $_[0], 'DE_EPAGES::Core::API::Iterator' ), 'Pull.Iterator' );
            return $i-- // undef;
        },
        Count => sub {
            ok( IsA( $_[0], 'DE_EPAGES::Core::API::Iterator' ), 'Count.Iterator' );
            return 1;
        },
        Handle => sub {
            ok( IsA( $_[0], 'DE_EPAGES::Core::API::Iterator' ), 'Handle.Iterator' ),
            is( $_[1], 1, 'Handle.Value' );
        }
    );
    is( 1, $TestIterator->count, 'TestIterator.count' );
    is( 1, scalar <$TestIterator>, 'scalar TestIterator' );

    my $SquareIterator = ArrayIterator( [ 1..9 ], sub { $_[1] * $_[1] } );
    is( $SquareIterator->next, 1, 'first square' );
    is( $SquareIterator->next, 4, 'second square' );
    is( $SquareIterator->next, 9, 'third square' );
    my $Count = 3;
    $Count++ while <$SquareIterator>;
    is( $Count, 9, 'SquareIterator' );

    my $EvenIterator = ArrayIterator( [ 1..9 ], sub {
        my( $Iterator, $Value ) = @_;
        return $Value % 2 ? $Iterator->next : $Value;
    });
    is( $EvenIterator->next, 2, 'first even' );
    is( $EvenIterator->next, 4, 'second even' );
    is( $EvenIterator->next, 6, 'third even' );
    $Count = 3;
    $Count++ while <$EvenIterator>;
    is( $Count, 4, 'EvenIterator' );

    my $P3Iterator = ArrayIterator( [ 1..9 ], sub {
        my( $Iterator, $Value ) = @_;
        return $Value % 3 ? $Iterator->next : $Value;
    });
    is( $P3Iterator->next, 3, 'first P3' );
    is( $P3Iterator->next, 6, 'second P3' );
    is( $P3Iterator->next, 9, 'third P3' );
    ok( !defined $P3Iterator->next, 'P3Iterator' );

    $i = 1;
    my $Max = 1000;
    my $DynamicIterator = DE_EPAGES::Core::API::Iterator->new(
        Pull => sub { return $i <= $Max ? $i++ : undef },
        Count => sub { return $Max },
    );
    is( $DynamicIterator->count, 1000, 'DynamicIterator.count' );
    $Sum = 0;
    $Sum += $_ while <$DynamicIterator>;
    is( $Sum, 500500, 'DynamicIterator' );

    $i = 1;
    $Count = 1000;
    my $DynamicSquareIterator = DE_EPAGES::Core::API::Iterator->new(
        Pull => sub { return $i <= $Count ? $i++ : undef },
        Count => $Count,
        Handle => sub { $_[1] * $_[1] }
    );
    is( $DynamicSquareIterator->count, 1000, 'DynamicSquareIterator.count' );
    $Sum = 0;
    $Sum += $_ while <$DynamicSquareIterator>;
    is( $Sum, 333833500, 'DynamicSquareIterator' );

    my $WrapIterator = DE_EPAGES::Core::API::Iterator->wrap(
       Source => ArrayIterator( [ 1..9 ] ),
       Handle => sub { $_[1] * $_[1] }
    );
    $Sum = 0;
    $Sum += $_ while <$WrapIterator>;
    is( $Sum, 285, 'WrapIterator' );

    # test Destroy
    my $Destroyed = 0;
    {
        my $WrapIterator = DE_EPAGES::Core::API::Iterator->new(
            Pull => sub { die 'should not be called by this test' },
            Count => 1,
            Destroy => sub { $Destroyed = 1 },
        );
        ok !$Destroyed, 'Iterator object is still active';
    }
    ok $Destroyed, 'Destroy handler called when Iterator goes out of scope';
});
